@@ -1,4 +1,4 @@
-# $Id: tableparser.py 9038 2022-03-05 23:31:46Z milde $
+# $Id: tableparser.py 10136 2025-05-20 15:48:27Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -17,8 +17,9 @@ and produce a well-formed data structure suitable for building a CALS table.
     `update_dict_of_lists()`: Merge two dictionaries containing list values.
 """
 
-__docformat__ = 'reStructuredText'
+from __future__ import annotations
 
+__docformat__ = 'reStructuredText'
 
 import re
 import sys
@@ -35,7 +36,7 @@ class TableMarkupError(DataError):
     from the table's start line.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         self.offset = kwargs.pop('offset', 0)
         DataError.__init__(self, *args)
 
@@ -143,7 +144,7 @@ class GridTableParser(TableParser):
 
     head_body_separator_pat = re.compile(r'\+=[=+]+=\+ *$')
 
-    def setup(self, block):
+    def setup(self, block) -> None:
         self.block = block[:]           # make a copy; it may be modified
         self.block.disconnect()         # don't propagate changes to parent
         self.bottom = len(block) - 1
@@ -190,7 +191,7 @@ class GridTableParser(TableParser):
         if not self.check_parse_complete():
             raise TableMarkupError('Malformed table; parse incomplete.')
 
-    def mark_done(self, top, left, bottom, right):
+    def mark_done(self, top, left, bottom, right) -> None:
         """For keeping track of how much of each text column has been seen."""
         before = top - 1
         after = bottom - 1
@@ -198,7 +199,7 @@ class GridTableParser(TableParser):
             assert self.done[col] == before
             self.done[col] = after
 
-    def check_parse_complete(self):
+    def check_parse_complete(self) -> bool:
         """Each text column should have been completely seen."""
         last = self.bottom - 1
         for col in range(self.right):
@@ -372,7 +373,7 @@ class SimpleTableParser(TableParser):
     head_body_separator_pat = re.compile('=[ =]*$')
     span_pat = re.compile('-[ -]*$')
 
-    def setup(self, block):
+    def setup(self, block) -> None:
         self.block = block[:]           # make a copy; it will be modified
         self.block.disconnect()         # don't propagate changes to parent
         # Convert top & bottom borders to column span underlines:
@@ -386,7 +387,7 @@ class SimpleTableParser(TableParser):
         self.rowseps = {0: [0]}
         self.colseps = {0: [0]}
 
-    def parse_table(self):
+    def parse_table(self) -> None:
         """
         First determine the column boundaries from the top border, then
         process rows.  Each row may consist of multiple lines; accumulate
@@ -459,7 +460,7 @@ class SimpleTableParser(TableParser):
             i += 1
         return cells
 
-    def parse_row(self, lines, start, spanline=None):
+    def parse_row(self, lines, start, spanline=None) -> None:
         """
         Given the text `lines` of a row, parse it and append to `self.table`.
 
@@ -529,7 +530,7 @@ class SimpleTableParser(TableParser):
                 self.table[first_body_row:])
 
 
-def update_dict_of_lists(master, newdata):
+def update_dict_of_lists(master, newdata) -> None:
     """
     Extend the list values of `master` with those from `newdata`.
 
